# Autogenerated by nbdev

d = { 'settings': { 'branch': 'main',
                'doc_baseurl': '/cosette',
                'doc_host': 'https://AnswerDotAI.github.io',
                'git_url': 'https://github.com/AnswerDotAI/cosette',
                'lib_path': 'cosette'},
  'syms': { 'cosette.core': { 'cosette.core.Chat': ('core.html#chat', 'cosette/core.py'),
                              'cosette.core.Chat.__call__': ('core.html#chat.__call__', 'cosette/core.py'),
                              'cosette.core.Chat.__init__': ('core.html#chat.__init__', 'cosette/core.py'),
                              'cosette.core.Chat.structured': ('core.html#chat.structured', 'cosette/core.py'),
                              'cosette.core.Chat.use': ('core.html#chat.use', 'cosette/core.py'),
                              'cosette.core.ChatCompletion._repr_markdown_': ( 'core.html#chatcompletion._repr_markdown_',
                                                                               'cosette/core.py'),
                              'cosette.core.Client': ('core.html#client', 'cosette/core.py'),
                              'cosette.core.Client.__call__': ('core.html#client.__call__', 'cosette/core.py'),
                              'cosette.core.Client.__init__': ('core.html#client.__init__', 'cosette/core.py'),
                              'cosette.core.Client._r': ('core.html#client._r', 'cosette/core.py'),
                              'cosette.core.CompletionUsage.__add__': ('core.html#completionusage.__add__', 'cosette/core.py'),
                              'cosette.core.CompletionUsage.__repr__': ('core.html#completionusage.__repr__', 'cosette/core.py'),
                              'cosette.core._mock_id': ('core.html#_mock_id', 'cosette/core.py'),
                              'cosette.core.call_func_openai': ('core.html#call_func_openai', 'cosette/core.py'),
                              'cosette.core.can_set_system_prompt': ('core.html#can_set_system_prompt', 'cosette/core.py'),
                              'cosette.core.can_set_temperature': ('core.html#can_set_temperature', 'cosette/core.py'),
                              'cosette.core.can_stream': ('core.html#can_stream', 'cosette/core.py'),
                              'cosette.core.contents': ('core.html#contents', 'cosette/core.py'),
                              'cosette.core.find_block': ('core.html#find_block', 'cosette/core.py'),
                              'cosette.core.get_stream': ('core.html#get_stream', 'cosette/core.py'),
                              'cosette.core.mk_openai_func': ('core.html#mk_openai_func', 'cosette/core.py'),
                              'cosette.core.mk_tool_choice': ('core.html#mk_tool_choice', 'cosette/core.py'),
                              'cosette.core.mk_toolres': ('core.html#mk_toolres', 'cosette/core.py'),
                              'cosette.core.mock_tooluse': ('core.html#mock_tooluse', 'cosette/core.py'),
                              'cosette.core.usage': ('core.html#usage', 'cosette/core.py'),
                              'cosette.core.wrap_latex': ('core.html#wrap_latex', 'cosette/core.py')},
            'cosette.toolloop': {'cosette.toolloop.Chat.toolloop': ('toolloop.html#chat.toolloop', 'cosette/toolloop.py')}}}
